@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.6.0
+ *             Automatically Tuned Linear Algebra Software v3.8.4
  *                    (C) Copyright 2000 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,7 @@
 #include <string.h>
 #include "atlas_misc.h"
 #include "atlas_fopen.h"
+#include "atlas_prefetch.h"
 
 #define Mmin(x, y) ( (x) > (y) ? (y) : (x) )
 
@@ -201,7 +202,18 @@ double GetAvg(int n, double tolerance, double *mflop)
          }
       }
    }
+/*
+ * Not doing tolerance anymore, just take largest mflop rate if doing wall
+ * times, or median value if doing CPU
+ */
 
+#if 1
+   #ifdef WALL
+      tavg = mflop[0];
+   #else
+      tavg = mflop[n/2];
+   #endif
+#else
 /*
  * Throw out result if it is outside tolerance; rerun if two mflop not within
  * tolerance;  this code assumes n == 3
@@ -213,49 +225,53 @@ double GetAvg(int n, double tolerance, double *mflop)
    }
    else if (tolerance*mflop[2] < mflop[0]) tavg = (mflop[0] + mflop[1]) / 2.0;
    else tavg = (mflop[0] + mflop[1] + mflop[2]) / 3.0;
+#endif
 
    return(tavg);
 }
 
-void PutInstLogLine(FILE *fp, int muladd, int lat, int nb,
+void PutInstLogLine(FILE *fp, int muladd, int pfA, int lat, int nb,
                     int mu, int nu, int ku, int ForceFetch,
                     int ifetch, int nfetch, double mflop)
 {
-   fprintf(fp, "%6d  %3d %3d %3d %3d %3d  %5d  %5d  %5d  %7.2lf\n",
-           muladd, lat, nb, mu, nu, ku, ForceFetch, ifetch, nfetch, mflop);
+   fprintf(fp, "%6d  %3d %4d %3d %3d %3d %3d  %5d  %5d  %5d  %7.2lf\n",
+           muladd, lat, pfA, nb, mu, nu, ku, ForceFetch, ifetch, nfetch, mflop);
 }
-void PutInstLogFile(FILE *fp, int muladd, int lat, int nb,
+
+void PutInstLogFile(FILE *fp, int muladd, int pfA, int lat, int nb,
                     int mu, int nu, int ku, int ForceFetch,
                     int ifetch, int nfetch, double mflop)
 {
-   fprintf(fp, "MULADD  LAT  NB  MU  NU  KU  FFTCH  IFTCH  NFTCH    MFLOP\n");
-   PutInstLogLine(fp, muladd, lat, nb, mu, nu, ku, ForceFetch,
+   fprintf(fp,
+   "MULADD  LAT  PREF NB  MU  NU  KU  FFTCH  IFTCH  NFTCH    MFLOP\n");
+   PutInstLogLine(fp, muladd, pfA, lat, nb, mu, nu, ku, ForceFetch,
                   ifetch, nfetch, mflop);
 }
-void PutInstLogFile1(char *fnam, char pre, int muladd, int lat, int nb,
-                    int mu, int nu, int ku, int ForceFetch,
-                    int ifetch, int nfetch, double mflop)
+
+void PutInstLogFile1(char *fnam, char pre, int muladd, int pfA, int lat,
+                     int nb, int mu, int nu, int ku,
+                     int ForceFetch, int ifetch, int nfetch, double mflop)
 {
    FILE *fp;
 
    fp = fopen(fnam, "w");
    assert(fp);
-   PutInstLogFile(fp, muladd, lat, nb, mu, nu, ku, ForceFetch, ifetch, nfetch,
-                  mflop);
+   PutInstLogFile(fp, muladd, pfA, lat, nb, mu, nu, ku, ForceFetch, ifetch,
+                  nfetch, mflop);
    fclose(fp);
 }
 
-void GetInstLogLine(FILE *fp, int *muladd, int *lat, int *nb,
+void GetInstLogLine(FILE *fp, int *muladd, int *pfA, int *lat, int *nb,
                     int *mu, int *nu, int *ku, int *ForceFetch,
                     int *ifetch, int *nfetch, double *mflop)
 {
-   assert(fscanf(fp, " %d %d %d %d %d %d %d %d %d %lf\n",
-                 muladd, lat, nb, mu, nu, ku, ForceFetch,
-                 ifetch, nfetch, mflop) == 10);
+   assert(fscanf(fp, " %d %d %d %d %d %d %d %d %d %d %lf\n",
+                 muladd, lat, pfA, nb, mu, nu, ku, ForceFetch,
+                 ifetch, nfetch, mflop) == 11);
 }
 
-void GetInstLogFile(char *nam, char pre, int *muladd, int *lat, int *nb,
-                    int *mu, int *nu, int *ku, int *ForceFetch,
+void GetInstLogFile(char *nam, char pre, int *muladd, int *pfA, int *lat,
+                    int *nb, int *mu, int *nu, int *ku, int *ForceFetch,
                     int *ifetch, int *nfetch, double *mflop)
 {
    char ln[128];
@@ -265,7 +281,7 @@ void GetInstLogFile(char *nam, char pre, int *muladd, int *lat, int *nb,
    if (fp == NULL) fprintf(stderr, "file %s not found!!\n\n", nam);
    assert(fp);
    fgets(ln, 128, fp);
-   GetInstLogLine(fp, muladd, lat, nb, mu, nu, ku, ForceFetch,
+   GetInstLogLine(fp, muladd, pfA, lat, nb, mu, nu, ku, ForceFetch,
                   ifetch, nfetch, mflop);
    fclose(fp);
 }
@@ -354,7 +370,7 @@ int GetUserCase(char pre, int icase, int *iflag, int *mb, int *nb, int *kb,
 {
    int i, n, ID;
    char ln[512];
-   static char sMCC[128], sMMFLAGS[512];
+   static char sMCC[1024], sMMFLAGS[2048];
    FILE *fp;
 
    n = NumUserCases(pre);
@@ -719,15 +735,21 @@ double ummcase0
 )
 {
    char ln[512];
+   char ch;
    int i;
    double mf;
 
    if (!FileExists(outnam))
    {
       if (pre == 'c' || pre == 'z')
-         i = sprintf(ln, "make cmmucase mmrout=../CASES/%s csC=2 ", fnam);
-      else i = sprintf(ln, "make mmucase mmrout=../CASES/%s ", fnam);
-      if (MCC) i += sprintf(ln+i, "MCC=\"%s\" MMFLAGS=\"%s\" ", MCC, MMFLAGS);
+         i = sprintf(ln, "make cmmucase mmrout=CASES/%s csC=2 ", fnam);
+      else i = sprintf(ln, "make mmucase mmrout=CASES/%s ", fnam);
+      if (MCC)
+      {
+         ch = (pre == 'c' || pre == 's') ? 'S' : 'D';
+         i += sprintf(ln+i, "%cMC=\"%s\" %cMCFLAGS=\"%s\" ",
+                      ch, MCC, ch, MMFLAGS);
+      }
       i += sprintf(ln+i, "casnam=%s ", outnam);
       i += sprintf(ln+i, "pre=%c muladd=%d lat=%d M=%d N=%d K=%d mb=%d nb=%d kb=%d mu=%d nu=%d ku=%d lda=%d ldb=%d ldc=%d ",
                    pre, muladd, lat, M, N, K, mb, nb, kb, mu, nu, ku,
@@ -984,11 +1006,16 @@ int ummtstcase0
 {
    char ln[512];
    int i;
+   char ch;
 
    if (pre == 'c' || pre == 'z')
-      i = sprintf(ln, "make cmmutstcase mmrout=../CASES/%s csC=2 ", fnam);
-   else i = sprintf(ln, "make mmutstcase mmrout=../CASES/%s ", fnam);
-   if (MCC) i += sprintf(ln+i, "MCC=\"%s\" MMFLAGS=\"%s\" ", MCC, MMFLAGS);
+      i = sprintf(ln, "make cmmutstcase mmrout=CASES/%s csC=2 ", fnam);
+   else i = sprintf(ln, "make mmutstcase mmrout=CASES/%s ", fnam);
+   if (MCC)
+   {
+      ch = (pre == 's' || pre == 'c') ? 'S' : 'D';
+      i += sprintf(ln+i, "%cMC=\"%s\" %cMCFLAGS=\"%s\" ", ch, MCC, ch, MMFLAGS);
+   }
    i += sprintf(ln+i, "pre=%c muladd=%d lat=%d M=%d N=%d K=%d mb=%d nb=%d kb=%d mu=%d nu=%d ku=%d lda=%d ldb=%d ldc=%d ",
                 pre, muladd, lat, M, N, K, mb, nb, kb, mu, nu, ku,
                 lda, ldb, ldc);
@@ -997,12 +1024,71 @@ int ummtstcase0
    return(system(ln) == 0);
 }
 
+static int Mylcm(const int M, const int N)
+/*
+ * Returns least common multiple (LCM) of two positive integers M & N by
+ * computing greatest common divisor (GCD) and using the property that
+ * M*N = GCD*LCM.
+ */
+{
+   register int tmp, max, min, gcd=0;
+
+   if (M != N)
+   {
+      if (M > N) { max = M; min = N; }
+      else { max = N; min = M; }
+      if (min > 0)  /* undefined for negative numbers */
+      {
+         do  /* while (min) */
+         {
+            if ( !(min & 1) ) /* min is even */
+            {
+               if ( !(max & 1) ) /* max is also even */
+               {
+                  do
+                  {
+                     min >>= 1;
+                     max >>= 1;
+                     gcd++;
+                     if (min & 1) goto MinIsOdd;
+                  }
+                  while ( !(max & 1) );
+               }
+               do min >>=1 ; while ( !(min & 1) );
+            }
+/*
+ *          Once min is odd, halve max until it too is odd.  Then, use
+ *          property that gcd(max, min) = gcd(max, (max-min)/2)
+ *          for odd max & min
+ */
+MinIsOdd:
+            if (min != 1)
+            {
+               do  /* while (max >= min */
+               {
+                  max -= (max & 1) ? min : 0;
+                  max >>= 1;
+               }
+               while (max >= min);
+            }
+            else return( (M*N) / (1<<gcd) );
+            tmp = max;
+            max = min;
+            min = tmp;
+         }
+         while(tmp);
+      }
+      return( (M*N) / (max<<gcd) );
+   }
+   else return(M);
+}
+
 int GetUserNB(char pre, int NB, int mb, int nb, int kb)
 /*
  * Given ATLAS's preference for NB, finds a good nb based on user's input
  */
 {
-   int i, nelt;
+   int i, nelt, mult;
 
    if (mb < 0)
    {
@@ -1018,12 +1104,13 @@ int GetUserNB(char pre, int NB, int mb, int nb, int kb)
    if (!nb) nb = 1;
    if (!mb) mb = 1;
    if (!kb) kb = 1;
+   mult = Mylcm(mb,nb);
+   mult = Mylcm(kb,mult);
    if ( (NB%mb == 0) && (NB%nb == 0) && (NB%kb == 0) ) return(NB);
    nelt = L1Elts(pre, 128*1024);
-   for (i=NB; (i%nb || i%mb || i%kb) && i*i < nelt; i += 4);
-   if ( (i%mb == 0) && (i%nb == 0) && (i%kb == 0) ) return(i);
-   for (i=NB-4; i > 16 && (i%nb || i%mb || i%kb); i -= 4);
-   if ( (i%mb == 0) && (i%nb == 0) && (i%kb == 0) ) return(i);
+   i = ((NB+mult-1)/mult)*mult;
+   if (i*i < nelt) return(i);
+   else return((NB/mult)*mult);
    return(0);
 }
 
@@ -1149,66 +1236,6 @@ void GetFlags(int nargs, char **args, char *pre, int *nb0,
    }
 }
 
-
-static int Mylcm(const int M, const int N)
-/*
- * Returns least common multiple (LCM) of two positive integers M & N by
- * computing greatest common divisor (GCD) and using the property that
- * M*N = GCD*LCM.
- */
-{
-   register int tmp, max, min, gcd=0;
-
-   if (M != N)
-   {
-      if (M > N) { max = M; min = N; }
-      else { max = N; min = M; }
-      if (min > 0)  /* undefined for negative numbers */
-      {
-         do  /* while (min) */
-         {
-            if ( !(min & 1) ) /* min is even */
-            {
-               if ( !(max & 1) ) /* max is also even */
-               {
-                  do
-                  {
-                     min >>= 1;
-                     max >>= 1;
-                     gcd++;
-                     if (min & 1) goto MinIsOdd;
-                  }
-                  while ( !(max & 1) );
-               }
-               do min >>=1 ; while ( !(min & 1) );
-            }
-/*
- *          Once min is odd, halve max until it too is odd.  Then, use
- *          property that gcd(max, min) = gcd(max, (max-min)/2)
- *          for odd max & min
- */
-MinIsOdd:
-            if (min != 1)
-            {
-               do  /* while (max >= min */
-               {
-                  max -= (max & 1) ? min : 0;
-                  max >>= 1;
-               }
-               while (max >= min);
-            }
-            else return( (M*N) / (1<<gcd) );
-            tmp = max;
-            max = min;
-            min = tmp;
-         }
-         while(tmp);
-      }
-      return( (M*N) / (max<<gcd) );
-   }
-   else return(M);
-}
-
 int FindBestNB(pre, icase)
 {
    double mf, mfb;
@@ -1392,7 +1419,7 @@ void RunTimes(char pre)
    {
       sprintf(ln, "res/%cMMRES", pre);
       if (!FileExists(ln)) sprintf(ln, "res/%cgMMRES", pre);
-      GetInstLogFile(ln, pre, &j, &j, &nb, &j, &j, &j, &j, &j, &j, &mf);
+      GetInstLogFile(ln, pre, &j, &j, &j, &nb, &j, &j, &j, &j, &j, &j, &mf);
       FindUMM(pre, nb);
    }
 }
@@ -1423,7 +1450,7 @@ void GetuMMRES(char pre)
    {
       sprintf(ln, "res/%cMMRES", pre);
       if (!FileExists(ln)) sprintf(ln, "res/%cgMMRES", pre);
-      GetInstLogFile(ln, pre, &j, &j, &nb, &j, &j, &j, &j, &j, &j, &mf);
+      GetInstLogFile(ln, pre, &j, &j, &j, &nb, &j, &j, &j, &j, &j, &j, &mf);
       FindUMM(pre, nb);
    }
 }
