@@ -209,8 +209,8 @@ double GetMflop0(char pre, int ID, char *rout, int incX, int incY,
    {
       i = sprintf(ln, "make %ciamaxcase N=%d urout=%s fout=\"-f %s\" ",
                   pre, N, rout, fnam);
-      if (cc) i += sprintf(ln+i, "UCC=\"%s\" ", cc);
-      if (ccflags) i += sprintf(ln+i, "UCCFLAGS=\"%s\" ", ccflags);
+      if (cc) i += sprintf(ln+i, "%cUCC=\"%s\" ", pre, cc);
+      if (ccflags) i += sprintf(ln+i, "%cUCCFLAGS=\"%s\" ", pre, ccflags);
       i += sprintf(ln+i, "opt=\"");
       if (incX != 1) i += sprintf(ln+i, "-X %d ", incX);
       i += sprintf(ln+i, "\"\n");
@@ -508,8 +508,8 @@ FILENODE *TstFile(char pre, FILENODE *fbase, int *N)
       i = sprintf(ln, "make %ciamaxtest urout=%s opt=\"", pre, fp->rout);
       if (fp->incX == 0) i += sprintf(ln+i, " -X 4 1 -1 2 -3");
       i += sprintf(ln+i, "\" ");
-      if (fp->cc) i += sprintf(ln+i, "UCC=\"%s\" ", fp->cc);
-      if (fp->ccflags) i += sprintf(ln+i, "UCCFLAGS=\"%s\" ", fp->ccflags);
+      if (fp->cc) i += sprintf(ln+i, "%cUCC=\"%s\" ", pre, fp->cc);
+      if (fp->ccflags) i += sprintf(ln+i, "%cUCCFLAGS=\"%s\" ",pre,fp->ccflags);
       sprintf(ln+i, "\n");
       fprintf(stderr, "TST: %s", ln);
       #if 1
@@ -603,14 +603,15 @@ void GenMakefile(char pre, int n, int *ix, int *iy, int *ia, int *ib,
 
    sprintf(ln, "ATL_%c%s", pre, rout);
    fprintf(fpout, "%s.o : %s.c\n", ln, ln);
-   fprintf(fpout, "\t$(CC) $(CCFLAGS) -D%s -o $@ -c %s.c\n", typ, ln);
+   fprintf(fpout, "\t$(ICC) $(ICCFLAGS) -D%s -o $@ -c %s.c\n", typ, ln);
    for (i=0; i < n; i++)
    {
       nam = GetNam(pre, ia[i], ib[i], ix[i], iy[i]);
       cc = bp[i]->cc;
       ccflags = bp[i]->ccflags;
-      if (!cc) cc = "$(CC)";
-      if (!ccflags) ccflags = "$(CCFLAG0)";
+      if (!cc) cc = (pre == 's' || pre == 'c') ? "$(SKC)" : "$(DKC)";
+      if (!ccflags)
+         ccflags = (pre == 's' || pre == 'c') ? "$(SKCFLAGS)":"$(DKCFLAGS)";
       fprintf(fpout, "%s.o : %s.c\n", nam, nam);
       fprintf(fpout, "\t%s %s $(CDEFS) -D%s -o $@ -c %s.c\n",
               cc, ccflags, typ, nam);
@@ -681,7 +682,7 @@ void DumpFile(char *fnam, FILE *fpout)
 {
    FILE *fpin;
    char ln[512];
-   sprintf(ln, "../IAMAX/%s", fnam);
+   sprintf(ln, "IAMAX/%s", fnam);
    fpin = fopen(ln, "r");
    assert(fpin);
    while(fgets(ln, 512, fpin)) fputs(ln, fpout);
